#include "simple_uart.h"
#include "boards.h"

static const char digits[] = "0123456789abcdef";

static char* itoa(int value, int base)
{

    static char buffer[32+1];

    char* p1 = buffer;
    char* p2;
    const int sign = (value >= 0) ? 1 : -1;

    if (base <  2 || base > 16) { return "error !"; }

    value = sign * value;
    do
    {
       *p1++ = digits[value % base];
    } while (value /= base);

    if (sign == -1) *p1++ = '-';
    *p1-- = 0;

    // reverse string
    for (p2 = buffer; p2 < p1; p2++, p1--)
    {
       char temp = *p1;
       *p1 = *p2;
       *p2 = temp;
       //*p1 = *p1 ^ *p2; // too cool !!!!!
       //*p2 = *p2 ^ *p1;
       //*p1 = *p1 ^ *p2;
    }

    return buffer;
}

void uart_init(void)
{
    simple_uart_config(RTS_PIN_NUMBER, TX_PIN_NUMBER, CTS_PIN_NUMBER, RX_PIN_NUMBER, HWFC);
}

void uart_print(const char* str)
{
   simple_uart_putstring((uint8_t*) str);
}

void uart_println(const char* str)
{
   uart_print(str);
   uart_print("\n\r");
}

void uart_print_dec(int value)
{
   uart_print(itoa(value, 10));
}

void uart_print_hex(int value)
{
   uart_print(itoa(value, 16));
   uart_print("H");
}

void uart_print_ble_evt(int evt)
{
   char* desc = 0;

   switch (evt)
   {
      case 0x10: desc = "BLE_GAP_EVT_CONNECTED"; break;
      case 0x11: desc = "BLE_GAP_EVT_DISCONNECTED"; break;
      case 0x12: desc = "BLE_GAP_EVT_CONN_PARAM_UPDATE"; break;
      case 0x13: desc = "BLE_GAP_EVT_SEC_PARAMS_REQUEST"; break;
      case 0x14: desc = "BLE_GAP_EVT_SEC_INFO_REQUEST"; break;
      case 0x15: desc = "BLE_GAP_EVT_PASSKEY_DISPLAY"; break;
      case 0x16: desc = "BLE_GAP_EVT_AUTH_KEY_REQUEST"; break;
      case 0x17: desc = "BLE_GAP_EVT_AUTH_STATUS"; break;
      case 0x18: desc = "BLE_GAP_EVT_CONN_SEC_UPDATE"; break;
      case 0x19: desc = "BLE_GAP_EVT_TIMEOUT"; break;
      case 0x1A: desc = "BLE_GAP_EVT_RSSI_CHANGED"; break;

      case 0x30: desc = "SD_BLE_GATTC_PRIMARY_SERVICES_DISCOVER"; break;
      case 0x31: desc = "SD_BLE_GATTC_RELATIONSHIPS_DISCOVER"; break;
      case 0x32: desc = "SD_BLE_GATTC_CHARACTERISTICS_DISCOVER"; break;
      case 0x33: desc = "SD_BLE_GATTC_DESCRIPTORS_DISCOVER"; break;
      case 0x34: desc = "SD_BLE_GATTC_CHAR_VALUE_BY_UUID_READ"; break;
      case 0x35: desc = "SD_BLE_GATTC_READ"; break;
      case 0x36: desc = "SD_BLE_GATTC_CHAR_VALUES_READ"; break;
      case 0x37: desc = "SD_BLE_GATTC_WRITE"; break;
      case 0x38: desc = "SD_BLE_GATTC_HV_CONFIRM"; break;

      default: break;
   }

   if (desc)
   {
      uart_print(desc);
   }
   else
   {
      uart_print("BLE_EVT: ");
      uart_print_hex(evt);
      uart_println(" no yet described or unknown !");
   }
}



